/* 
 * Autor:	Adam Bartczak, 5TIZ
 * Temat:	Biblioteka STL, Opis algorytmw : count count_if
 *
 * count count_if nale do algorytmw niemodyfikujcych biblioteki STL
 * 
 * Autor:	Adam Bartczak, 5TIZ
 * Temat:	Biblioteka STL, Opis algorytmw : count count_if
 *
 * count count_if nale do algorytmw niemodyfikujcych biblioteki STL
 *
 * template <class InputIterator, class T>
 *	 ptrdiff_t count ( InputIterator first, InputIterator last, const T& value );
 *
 * ptrdiff_t count_return = count(<element od ktrego zaczynamy wyszukiwanie>, 
 *						<element na ktrym koczymy wyszukiwanie>, <warot szukana>);
 *
 * Algorytm count oblicza licz elementw w kontenerze z zakresu [first, last) ktre pasuj do warotoci value,
 * nastpnie zwraca liczb pasujcych elementw. 
 *
 * template <class InputIterator, class Predicate>
 *	 ptrdiff_t count_if ( InputIterator first, InputIterator last, Predicate predict );
 *
 * ptrdiff_t count_if_return = count_if(<element od ktrego zaczynamy wyszukiwanie>,
 *						<element na ktrym koczymy wyszukiwanie>, <predykat do spenienia>);
 *
 * Algorytm count_if oblicza liczb elementw w kontenerze z zakresu [first, last) speniajcych predykat predict 
 * przekazany jako obiekt funkcyjny, nastpnie zwraca liczb pasujcych elementw.
 *
 * Przykad zastosowania algorytmw count oraz count_if oparty jest na wyszukiwaniu odpowiednich elementw  
 * (wciwych imion) wrd kontenera zawierajcego polskie imiona.
 */
 
 #include <iostream>
 #include <vector>
 #include <string>
 // biblioteka z algorytmami STL
 #include <algorithm>
 // biblioteka wykorzystywana do dzielenia napisw
 #include <boost/tokenizer.hpp>
  
 using namespace std;
 
 // funkcja wykrywajca polskie imiona kobiet
 bool  NameWoman(const string& str) {
    return (str[str.size()-1] == 'a' || str[str.size()-1] == 'A');
 }
 
 // funkcja wykrywajca polskie imiona mczyzn
 bool  NameMan(const string& str) {
    return (!NameWoman(str));
 }
 
 // funkcja wykrywajca imiona skadajce si z wicej ni 5 liter 
 bool  NameSize(const string& str) {
    return (str.size() > 5);
 }

 int main() {
	
    typedef vector<string> VectorString;
	
    typedef VectorString::iterator VectorStringIt;
  	
    // wektor imion
    VectorString vect;
  	
    // iteratory wskazujace na wektor imion vect
    VectorStringIt start, end;
  	
    // zmienna zawierajca liczb elementw pasujca do danego wzroca
    ptrdiff_t result;
  	
    // przykladowy napis ktory jest wypelniony polskimi imionami, odzielony znakami
    // ktre nie s literami alfabetu (separatorami). W imionach nie naley wstawia 
    // polskich liter (,,...), zamiast ich stosowa odpowiedniki (a,e,l...).
    const string STRING_NAMES = "Adam Michal Bartek,   Adam . Ania ,/ EwA";
  	
    // przykladowy wzorzec imienia dla funkcji count
    const string VALUE_COUNT = "Adam";

    // dzielenie napisu na poszczeglne imiona, nastpnie przypisywanie kadego z nich do wektora imion vect
    boost::tokenizer<> tok(STRING_NAMES);
    for(boost::tokenizer<>::iterator beg=tok.begin(); beg!=tok.end();++beg){
        vect.push_back(*beg);
    }
    
    // przypisanie iteratorom start i end wartoci pocztkowych
    start = vect.begin();  
    end = vect.end();
    
    // liczba wystapien imienia zawartego w zmiennej VALUE_COUNT (zastosowanie algorymu count)
    result = count(start, end, VALUE_COUNT) ;

    cout << "Liczba wystapien imienia " << VALUE_COUNT << ": " << result << endl;
    
    // liczba imion eskich w wektorze imion vect (zastosowanie algorymu count_if)
    result = count_if(start, end, NameWoman) ;
    
    cout << "Liczba Kobiet: " << result << endl;
    
    // liczba imion mskich w wektorze imion vect (zastosowanie algorymu count_if)
    result = count_if(start, end, NameMan) ;
    
    cout << "Liczba Mezczyzn: " << result << endl;
    
    // liczba imion zawierajce wicej ni 5 liter (zastosowanie algorymu count_if)
    result = count_if(start, end, NameSize) ;
    
    cout << "Liczba slow o liczbie liter > 5: " << result << endl; 
	
    return 0;	 
 }